package examples.xml.adapters;

import com.ibm.mqe.*;
import java.io.*;
import com.ibm.mqe.xml.*;
import com.ibm.mqe.messagestore.*;

// Version mqe_xml_supportpac/source/examples/xml/adapters/MQeXMLFieldsAdapter.java, MQeXML, la000 1.3
/* -----------------------------------------------------------------------------*/
/*                   MQSeries Everyplace for Multiplatforms                     */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5765-E63                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 1999                                                 */
/* -----------------------------------------------------------------------------*/

/**
 * This class is an disk adapter example using the MQeMsgObject -> XML transformer
 */
public class MQeXMLFieldsAdapter extends com.ibm.mqe.adapters.MQeDiskFieldsAdapter 
{
	private MQeXMLConverter converter = null;

	
	/**
	 * The activate() method initialises a new MQeDiskFieldsAdapter object.
	 * Note that this call also sets the QOS Size parameter to be the size of
	 * the file or MQeFields object.
	 *
	 * @param fileDesc String - The directory path to be opened.
	 * Note: If you want to open a file the name of the file
	 * will be specified on the open. Just the path to the file here.
	 *
	 * @param param Object - Sets the adapter's parameter value.
	 * Can supply an MQeAttribute object here that will cause the data
	 * to be encoded on disk. See the writeObject/readObject methods.
	 *
	 * @param options Object - String that specifies the open options:
	 *        MQe_Adapter_READ   - read  the file
	 *        MQe_Adapter_WRITE  - write the file
	 *        MQe_Adapter_UPDATE - read and write the file
	 *
	 * @param lrecl Not used.
	 * @param norec Not used.
	 */
	public void   activate( String  fileDesc,
							Object  param,
							Object  options,
							int     lrecl,
							int     noRec ) throws Exception {

		// Calling our parent's activate method
		super.activate(fileDesc, param,options,lrecl,noRec);

		//create an instance of the transformer class
		Errorhandler eh = new Default_Errorhandler();
		converter = new MQeXMLConverter(eh);
	}
	/**
	 * Reads an MQeFields object from the current file.
	 *
	 * @param opt Object MQeFields object that can be used to
	 * provide a filter within the directory object. Otherwise null.
	 *
	 * @return Object - The MQeFields object that was read.
	 */
	public synchronized Object readObject( Object opt ) throws Exception 
	{
		// First check that the file or directory is opened for reading.
		if ( reading ) 
		{
			// Does the user want us to match a supplied MQeFields?
			// If so, we need to
			// 1. list all files in the directory
			// 2. Read each file in turn and restore as a Fields object
			// 3. Try an equality check - if equal then return that object
			if ( opt instanceof MQeFields ) 
			{
				String List[] = listMatchingFiles();
				MQeFields Fields = null;

				// Go through the list of files.
				for ( int i = 0; i < List.length; i = i + 1 )
					try 
					{
						// Open each file and read the MQeFields object.
						fileName = List[i];
						open( fileName );
						Fields = (MQeFields) readObject( null );

						// Compare the MQeFields just read with
						// the one the user specified.
						if ( Fields.equals( (MQeFields) opt ) )
						{
							trace( 113, fileName );
							return( Fields );
						}
					} 
					catch ( Exception e ) 
					{
						// Something went wrong. Print a stack trace
						// (this may be removed).
						if ( MQe.debugExcept && (! (e instanceof MQeException)) )
							e.printStackTrace();
					}
				throw new MQeException( Except_NotFound, "No match" );
			}

			// If we get here then the user didn't specify a match.
			// Just open the file and read the bytes from disk.
			File diskFile = new File( filePath + fileName );

			// Read the data bytes into an array.
			byte data[]   = new byte[(int) diskFile.length( )];
			FileInputStream InputFile = new FileInputStream( diskFile );
			InputFile.read( data );

			// Close the file and update the bytes read field in the QOS object.
			InputFile.close( );
			qosIncrement( MQe.QoS_BytesRead, data.length );

			// Get the MQeFields from the bytes and return it
			String XML = new String(data);
			MQeFields FieldsObject = (MQeFields)converter.toMQeFields(XML);         	
			trace( 111, FieldsObject.toString() );
			return (FieldsObject);
		}

		// The file is not open for reading.
		else
			throw new MQeException( MQe.Except_NotSupported, "Not supported" );
	}
	/**
	 * Writes an MQeFields object to disk.
	 * @param opt Object not used.
	 * @param data Object MQeFields object to dump to disk.
	 */
	public synchronized void writeObject( Object  opt,
										  Object  data ) throws Exception {

		// Only allow the operation if the user supplied an MQeFields
		// object and the current file is available for writing.
		if ( writing && (data instanceof MQeFields) ) 
		{
			// Convert the string object into a XML byte array
			byte dump[] = converter.toXMLbyteArray((MQeFields)data);
			
			// Update the QOS size attribute with the file size
			// before opening the file and dumping the bytes out to it.
			qos.putInt( MQe.QoS_Size, dump.length );

			File diskFile = new File( filePath + fileName );
			FileOutputStream OutputFile = new FileOutputStream( diskFile );
			OutputFile.write( dump );

			// Synchronise the update with any cache stuff that may be around.
			OutputFile.getFD().sync();
			OutputFile.close();

			// Update the QOS object with the number of bytes written.
			qosIncrement( MQe.QoS_BytesWritten, dump.length );
			trace( 110, data.toString() );
		} 
		else 
		{
			// Thrown if the file is not available for writing or if
			// an MQeFields object was not supplied.
			throw new MQeException( MQe.Except_NotSupported, "Not supported" );
		}
	}
}
