package examples.xml.administration;

// Version mqe_xml_supportpac/source/examples/xml/administration/LocalXMLQueueCreator.java, MQeXML, la000 1.2
/*------------------------------------------------------------------------------*/
/*                    MQSeries Everyplace for Multiplatforms                    */
/*                                                                              */
/* Licensed Materials - Property of IBM                                         */
/*                                                                              */
/* 5765-E63                                                                     */
/*                                                                              */
/* (C) Copyright IBM Corp. 2001                                                 */
/*                                                                              */
/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
							  
/**
 * To add a local queue to a queue manager from the command-line.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.xml.administration.LocalXMLQueueCreator
 * &lt;queue_name&gt;
 * &lt;compressor&gt;
 * &lt;cryptor&gt;
 * &lt;authenticator&gt;
 * &lt;maxMsgSize&gt;
 * &lt;maxQDepth&gt;
 * &lt;queue_manager_name&gt;
 * &lt;store adapter&gt;
 * &lt;ini_file&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * <DT>queue_name</DT>
 * <DD>The name of the local queue to create on the target queue manager.</DD>
 * 
 * <DT>compressor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * compressor the queue should use to store messages on the disk.
 * Use "null" if you don't want a compressor.</DD>
 * 
 * <DT>cryptor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * cryptor the queue should use to store messages on the disk.
 * Use "null" if you don't want a cryptor.</DD>
 * 
 * <DT>authenticator</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * authenticator the queue should use to store messages on the disk.
 * Use "null" if you don't want an authenticator.</DD>
 * 
 * <DT>maxMsgSize</DT>
 * <DD>The size of the largest message (in bytes) which can be accepted on this queue.
 * This must be an integer number.
 * Specify "nolimit" if you want to allow any size of message to be put to the queue.
 * Specify "default" if you want to set this to the default value chosen by MQe.
 * </DD>
 * 
 * <DT>maxQDepth</DT>
 * <DD>The max number of messages allowed to be stored in the queue, before the queue is considered to be "full".
 * This must be an integer number.
 * Specify "nolimit" if you want to allow any limit to the depth of the queue
 * Specify "default" if you want to set this to the default value chosen by MQe.
 * </DD>
 * 
 * <DT>queue_manager_name</DT>
 * <DD>The name of the queue manager on which the new local queue will be defined.
 * This would normally be the name of the queue manager described by the .ini file</DD>
 *
 * <DT>store adapter</DT>
 * <DD>The name of the class which will be used as an disk adapter</DD>
 * 
 * <DT>ini_file</DT>
 * <DD>The fully-qualified path name of the .ini file to be used to send and receive the admin messages.</DD>
 * </DL>
 */
public class LocalXMLQueueCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool {

	public       static short version[]      = {1, 1, 0};


	public static final String syntax =
		  "Syntax: ...LocalQueueCreator <queue_name> <compressor> <cryptor> <authenticator>\n"
		+ "<maxMsgSize> <maxQDepth> <queue_manager_name> <store adapter> <ini_file>" 
		;

	/**
	 * Gathers command-line parameters into variables, and calls the method which creates the queue.
	 * 
	 * @param args
	 * 
	 * @exception Exception
	 */
	public void activate( String args[] ) throws Exception {
		String q = null ;
		String compressorClass = null ;
		String cryptorClass = null ;
		String authenticatorClass = null ;
		String maxMsgSize = null ;
		String maxQDepth = null ;
		String qmgr = null ;
		String adapter = null;
		String iniFile = null ;

		if( args.length < 9 ) {
			throw new Exception("Not enough parameters !\n"+syntax);
		}

		q                   = args[0].trim();
		compressorClass     = args[1].trim();
		cryptorClass        = args[2].trim();
		authenticatorClass  = args[3].trim();
		maxMsgSize          = args[4].trim();
		maxQDepth           = args[5].trim();
		qmgr                = args[6].trim();
		adapter				= args[7].trim();
		iniFile             = args[8].trim();

		System.out.println("i: Attempting to define a queue "+q+" on queue manager "+qmgr+" using the "+iniFile+" ini file.");

		// Access the local MQe queue manager...
		MQeClient client = new MQeClient( iniFile , true);

		// Access the queue manager created by the MQeClient class...
		MQeQueueManager myQMgr = client.queueManager;

		// Create the queue...
		createQueue( q 
				   , compressorClass 
				   , cryptorClass 
				   , authenticatorClass 
				   , maxMsgSize
				   , maxQDepth
				   , qmgr
				   ,adapter 
				   , myQMgr 
				   );

		// Close down the MQeClient
		client.close();
	}
	/**
	 * Creates an admin message which will create a queue, and sends it
	 * to the target queue manager.
	 * 
	 * @param q      The name of the local queue to create
	 * 
	 * @param compressorClass
	 *                   The name of the compression class to use (or alias which resolves to a class name)
	 *                   Should include the .class extension.
	 *                   Specify "null" if you don't want to use a compressor.
	 * 
	 * @param cryptorClass
	 *                   The name of the cryptor class to use (or alias which resolves to a class name)
	 *                   Should include the .class extension.
	 *                   Specify "null" if you don't want to use a cryptor.
	 * 
	 * @param authenticatorClass
	 *                   The name of the authenticator class to use (or alias which resolves to a class name)
	 *                   Should include the .class extension.
	 *                   Specify "null" if you don't want to use an authenticator
	 * 
	 * @param maxMsgSize The size of the largest message (in bytes) which can be accepted on this queue.
	 *                   This must be an integer number.
	 *                   Specify "nolimit" if you want to allow any size of message to be put to the queue.
	 *                   Specify "default" if you want to set this to the default value chosen by MQe.
	 * 
	 * @param maxQDepth The max number of messages allowed to be stored in the queue, before the queue is considered to be "full".
	 *                  This must be an integer number.
	 *                  Specify "nolimit" if you want to allow any limit to the depth of the queue
	 *                  Specify "default" if you want to set this to the default value chosen by MQe.
	 * 
	 * @param qmgr   The queue manager on which the local queue is created.
	 *
	 * @param adapter  The store adapter used for this queue, for instance examples.xml.adapters.MQeXMLFieldsAdapter
	 * 
	 * @param myQMgr A reference to an active MQe queue manager, which will be used to send and receive admin messages.
	 * 
	 * @exception Exception
	 */
	protected void createQueue( String q
							  , String compressorClass
							  , String cryptorClass
							  , String authenticatorClass
							  , String maxMsgSize
							  , String maxQDepth
							  , String qmgr
							  , String adapter
							  , MQeQueueManager myQMgr
							  ) throws Exception {

		System.out.println("Building the admin message to create a local queue "+q
						  +" on queue manager "+qmgr );
	 
		MQeQueueAdminMsg msg = new MQeQueueAdminMsg( qmgr
												   , q
												   );
		MQeFields params = new MQeFields();
	 
		// Fill the params with any other information we need for our queue definition.
		if(! cryptorClass.equals("null") ) {
			params.putAscii( msg.Queue_Cryptor       , cryptorClass );
		}
		if( ! authenticatorClass.equals("null") ) {
			params.putAscii( msg.Queue_Authenticator , authenticatorClass );
		}
		if( ! compressorClass.equals("null") ) {
			params.putAscii( msg.Queue_Compressor    , compressorClass );
		}

		if( maxMsgSize.equals("default") ) {
			// If we don't set the field to anything explicitly, it gets implicitly set
			// to a sensible default.
		} else {
			int value ;
			if( maxMsgSize.equals("nolimit") ) {
				value = msg.Queue_NoLimit ;
			} else {
				value = Integer.parseInt( maxMsgSize );
			}
			params.putInt( msg.Queue_MaxMsgSize , value );
		}

		if( maxQDepth.equals("default") ) {
			// If we don't set the field to anything explicitly, it gets implicitly set
			// to a sensible default.
		} else {
			int value ;
			if( maxQDepth.equals("nolimit") ) {
				value = msg.Queue_NoLimit ;
			} else {
				value = Integer.parseInt( maxQDepth );
			}
			params.putInt( msg.Queue_MaxQSize ,  value );
		}

		if( adapter.equals("default") ) {
			// If we don't set the field to anything explicitly, it gets implicitly set
			// to a sensible default.
		} else {
			String queue_store = myQMgr.getQueueStore();
			String queue_path = null;
			int index = queue_store.indexOf(':');
			if (index <0)
			{
				throw new MQeException("the queue store is not set properly");
			}
			else 
			{
				queue_path = queue_store.substring(index + 1);
			}
			String queue_adapter = adapter+":"+queue_path;
			params.putAscii(msg.Queue_FileDesc, queue_adapter);
		}
		
		// Say this admin message should create the queue...
		msg.create( params );
	 
		// Direct the admin message at the correct object...
		msg.setName( qmgr , q );
	 
		System.out.println("Built the admin message.");
	 
		// Send the message.
		MQeFields results = sendWaitMessage( myQMgr , qmgr , msg );
	}
	/**
	* Entry point from the command line.
	*/
	public static void main( String args[] ) {
		int rc = 0 ;
		LocalXMLQueueCreator me = new LocalXMLQueueCreator();
		try {
			me.activate( args );
		} catch( Exception e ) {
			System.out.println("Error: activate() failed. "+e.toString());
			rc = 1 ;
		}
		System.exit(rc); // exit,killing the trace window by force if necessary.
	}
}
